from WidgetFinder import WidgetFinder

from PythonUnit import TestCase
from PythonUnit import TestSuite

class FakeNode:
    def __init__(self, name):
        self.name = name

    def getNodeName(self):
        return self.name

    def absolute_url(self):
        return self.name

class FakeWidget:
    def __init__(self, nodename):
        self.nodename = nodename

    def _isNodeSupported(self, node, REQUEST):
        return 1


class WidgetFinderTestCase(TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_find_simple(self):
        wf = WidgetFinder()
        fooWidget = FakeWidget('foo')
        barWidget = FakeWidget('bar')
        foo2Widget = FakeWidget('foo')
        wf.addWidget(fooWidget)
        wf.addWidget(foo2Widget)
        wf.addWidget(barWidget)

        fooNode = FakeNode('foo')
        barNode = FakeNode('bar')
        unknownNode = FakeNode('unknown')
        
        self.should(wf.find(fooNode) == fooWidget)
        self.should(wf.find(barNode) == barWidget)
        self.should(wf.find(unknownNode) == None)

    def test_find_priority(self):
        wf = WidgetFinder(levels=2)

        fooWidget = FakeWidget('foo')
        barWidget = FakeWidget('bar')
        foo2Widget = FakeWidget('foo')
        wf.addWidget(fooWidget)
        wf.addWidget(foo2Widget, 1)
        wf.addWidget(barWidget, 0)
        
        fooNode = FakeNode('foo')
        barNode = FakeNode('bar')
        unknownNode = FakeNode('unknown')  

        self.should(wf.find(fooNode) == foo2Widget)
        self.should(wf.find(barNode) ==  barWidget)
        self.should(wf.find(unknownNode) == None)

        wf.clearLevel(1)

        self.should(wf.find(fooNode) == fooWidget)
        self.should(wf.find(barNode) == barWidget)

    def test_find_nodewidget(self):
        wf = WidgetFinder()

        fooWidget = FakeWidget('foo')
        barWidget = FakeWidget('bar')
        specialWidget = FakeWidget('foo')
        
        wf.addWidget(fooWidget)
        wf.addWidget(barWidget)

        fooNode = FakeNode('foo')
        barNode = FakeNode('bar')
        unknownNode = FakeNode('unknown')
        
        wf.addNodeWidget(specialWidget, fooNode)
        
        self.should(wf.find(fooNode) == specialWidget)
        self.should(wf.find(barNode) == barWidget)
        self.should(wf.find(unknownNode) == None)

        wf.clearNodeWidgets()

        self.should(wf.find(fooNode) == fooWidget)
        self.should(wf.find(barNode) == barWidget)
        self.should(wf.find(unknownNode) == None)
        
suite = TestSuite("WidgetFinder", WidgetFinderTestCase)

if __name__ == "__main__":
    from PythonUnit import TestRun

    t = TestRun("WigdetFinderTest.py")
    t.addSuite(suite)
    t.run()
    
