from Globals import HTMLFile
from Globals import MessageDialog
from Globals import Persistent

import OFS.SimpleItem            
import Acquisition               
import AccessControl.Role      

import string

from OFS import ZDOM

from WidgetFinder import WidgetFinder

class Error(Exception):
    pass

class ZWidgetManager(
    OFS.SimpleItem.Item,
    Persistent,
    Acquisition.Implicit,
    AccessControl.Role.RoleManager
    ):

    meta_type = "XMLWidgetManager"

    # the various tabs
    manage_options = (
        {'label' : 'Edit',     'action' : 'manage_main'},
        {'label' : 'Security', 'action' : 'manage_access'},
        )

    # display edit screen as main management screen
    manage_main = HTMLFile('widgetmanagerEdit', globals())

    # permissions
    __ac_permissions__ = (
        ('View Management Screens',
         ('manage_tabs',
          'manage_main',
          )
         ),

        ('Change permissions',
         ('manage_access',)),
        
        ('Access Contents Information',        
         ('__call__',
          )
         ),
        )

    def __init__(self, id, title):
        """Create a new widget manager.
        """
        self.id = id
        self.title = title
        self.widgetnames = []
        self.widgetfinder = WidgetFinder()
        self.defaultWidget = None
        
    def manage_edit(self, title, REQUEST=None):
        """Handle result from edit action.
        title - the new title of the manager
        REQUEST - contains other data
        """
        # set title
        self.title = title
                
        if REQUEST is not None:
            # extract list of widget names
            widgetnames = map(string.strip,
                              string.split(REQUEST['widgets'], "\n"))
            # filter out all empty names
            widgetnames = filter(lambda name: name != "", widgetnames)

            # add widgets, clearing previous widgets first 
            self.addWidgets(widgetnames)

            # set default widget
            if hasattr(self, REQUEST.default):
                self.defaultWidget = getattr(self, REQUEST.default)
            else:
                self.defaultWidget = None
            
        # done with editing
        if REQUEST is not None:
            message = "Properties changed"
            return self.manage_main(self, REQUEST,
                                    manage_tabs_message=message)
        
    def addWidgets(self, widgetnames):
        """Add a number of widgets to the widgetfinder.
        """
        # clear out widgets
        self.clearWidgets()
    
        # store the names in this object
        self.widgetnames = widgetnames

        # now add them to the widget finder
        widgetfinder = self.widgetfinder
        
        for widgetname in widgetnames:
            widget = getattr(self, widgetname)
            widgetfinder.addWidget(widget)

        self.widgetfinder = widgetfinder
        
    def clearWidgets(self):
        """Clear all widgets.
        """
        # remove widgetnames from this object
        self.widgetnames = []
        # clear all widgets from widgetfinder
        self.widgetfinder.clearLevel()
        self.widgetfinder = self.widgetfinder

    index_html = None

    def find(me, self, REQUEST):
        """Find the widget to draw with.
        """
        node = self
        widget = me.widgetfinder.find(node, REQUEST)
        # if we found the widget
        if widget:
            # get widget in acquisition context of node
            # need to use explicit acquisition, as Text nodes
            # do not support implicit acquisition
            widget = node.aq_acquire(widget.id)
            # make sure we can access widget from DTML
            REQUEST.set('xml_widget', widget)
            return widget
        else:
            # use default widget if available and if we're
            # looking up an element node
            if me.defaultWidget and node.getNodeType() == ZDOM.ELEMENT_NODE:
                return me.defaultWidget
            
            # otherwise, prepare hopefully informative error
            errtable = {
                'nodename': node.getNodeName(),
                'parentname': node.getParentNode().getNodeName()
                }
            
            errstr = "Could not find XMLWidget for '%(nodename)s' node in "\
                     "context of '%(parentname)s' node." % errtable
            # raise error
            raise Error, errstr
        
             
# form to add the WidgetManager to a folder
manage_addWidgetManagerForm = HTMLFile('widgetmanagerAdd', globals())

# adds WidgetManager to folder
def manage_addWidgetManager(self, id, title='', REQUEST=None):
    """Add manager to folder.
    """
    wm = ZWidgetManager(id, title)
    
    self._setObject(id, wm)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

    
